/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

/**
   CacheImage is an ImageProducer
   that can act as the source to a filter without having
   to return to the server for the pixel information.
 */

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.net.MalformedURLException;


public class CacheImage implements ImageProducer
{
	// The ColorModel, either direct or indexed
	private ColorModel colorModel;

	// The height and width of our cache image
	private int width, height;

	// The pixel information: bytes for indexed, ints for direct
	private byte[] pixelByte;
	private int[] pixelInt;

	// The set of consumers of this image producer
	private Vector consumers = new Vector();

	/**
	 * The constructor.
	 */
	public CacheImage(Image img, Component comp)
  {
		// First, catch your image...
		MediaTracker mt = new MediaTracker(comp);
		mt.addImage(img, 0);
		try {
			mt.waitForAll();

			// Set up the dimensions
			width = img.getWidth(comp);
			height = img.getHeight(comp);

			// Start to grab the information about the image
			PixelGrabber pg = new PixelGrabber(img, 0, 0,	width, height, false);
			pg.grabPixels();			// Gather the information
			colorModel = pg.getColorModel();// Record the colour model

			// Now store the pixel information (byte indexes
			// for index model, full RGB int values for direct)
			if (colorModel instanceof IndexColorModel)
				pixelByte = (byte[])pg.getPixels();
			else
				pixelInt = (int[])pg.getPixels();
		}
		catch(InterruptedException e) {
			System.out.println("InterruptedException: " + e);
		}
	}

	/**
	 * Add a consumer for out information.
	 *
	 * @param ic	A new ImageConsumer
	 */
	public void addConsumer(ImageConsumer ic) {
		if (!isConsumer(ic))
			consumers.addElement(ic);
	}

	/**
	 * Is this a current image consumer of ours?
	 *
	 * @param ic	The possible image consumer
	 * @returns		true if a current consumer, false otherwise
	 */
	public boolean isConsumer(ImageConsumer ic) {
		return consumers.contains(ic);
	}

	/**
	 * Remove a consumer.
	 * @param ic	A current ImageConsumer
	 */
	public void removeConsumer(ImageConsumer ic)
  {
		consumers.removeElement(ic);
	}

	/**
	 * Send the image information to all the consumers.
	 * @param ic	An ImageConsumer
	 */
	public void startProduction(ImageConsumer ic)
  {
		ImageConsumer tempCon;
		Enumeration enum;

		// If not already a consumer, make it one.
		addConsumer(ic);
		enum = consumers.elements();

		// Loop through all consumers...
		while (enum.hasMoreElements()) {
			tempCon = (ImageConsumer)(enum.nextElement());

			// Report size of image
			tempCon.setDimensions(width, height);

			// Give it hints as to the sort of image it is
			// (this helps the consumer optimize its use of the
			// image)
			tempCon.setHints(
				ImageConsumer.TOPDOWNLEFTRIGHT
				| ImageConsumer.COMPLETESCANLINES
				| ImageConsumer.SINGLEPASS
				| ImageConsumer.SINGLEFRAME
			);

			// Send pixels as either index bytes or RGB
			// ints back to the consumer
			if (colorModel instanceof IndexColorModel)
				tempCon.setPixels(0, 0, width, height, colorModel, pixelByte, 0, width);
			else
				tempCon.setPixels(0, 0, width, height, colorModel, pixelInt, 0, width);

			// Tell it that's the full information
			tempCon.imageComplete(ImageConsumer.STATICIMAGEDONE);
		}

//		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	}

	/**
	 * We need this method to say we are fully implementing the
	 * ImageProducer interface.
	 * @param ic	An ImageConsumer
	 */
	public void requestTopDownLeftRightResend(ImageConsumer ic)
  {
		return;
	}
}
