/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.util.Vector;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.IllegalArgumentException;


/**
 * TabPanel is a Panel extension which provides for
 * a tabbed dialog effect. It provides the visual aspect of
 * tabs and allows the programmer to decide what action to take
 * when a tab is activated. It can be used directly or extended.
 * When extending from TabPanel be sure to super()
 * during construction and to super.handleEvent(evt) from
 * handleEvent if you override it.
 * <p>
 * addTab():     to add a new tab to the panel.
 * setLabel()    to update an existing tab.
 * removeTab()   to remove tabs, or removeAllTabs().
 * setEnabled(): to enable or disable a tab.
 * select():     to show (activate) a tab


   DGS
   Rajout CUTTED/SLANTED
 */
public class TabPanel extends Panel
{
    /**
     * Position constant indicating tabs are to be put at the top of this panel.
     */
    public static final int TOP = 0;
    public static final int BOTTOM = 1;

    /**
     * Style constant indicating tabs are to have rounded corners.
     */
    public static final int ROUNDED = 0;
    public static final int CUTTED = 1;
    public static final int SLANTED = 2;

    public static final int NONE = 0;
    public static final int RAISED = 1;
    public int bevel = RAISED;

    Vector vPanels;
    String[] labels = null;
    private Vector vEnabled;
    protected Vector vPolys; // one polygon for each tab.

    // Zero-relative index of currently shown tab. -1 if no tabs exist or are shown.
    protected int curIndex = -1;

    /**
      * A flag indicating repaints should be suppress during the setLabel()
      * and setEnabled() methods.
      */
     transient protected boolean suppressRepaints = false;
    /**
      * The zero-relative index of the first visible tab.
      */
     protected int firstVisibleTab = 0;
     /**
       * A left-pointing button shown when there are too many tabs to display.
       */
     protected DirButton dbLeft;
     /**
       * A right-pointing button shown when there are too many tabs to display.
       */
     protected DirButton dbRight;
     /**
       * An empty polygon.
       */
     protected Polygon nullPoly;
     /**
      * A Vector of tab label Strings.
      */
     protected Vector vLabels;
    /**
     * Color used in drawing of the border.
     */
     protected Color borderDarkerColor	= null;
    /**
     * Color used in drawing of the border.
     */
     protected Color borderLightColor	= null;
    /**
     * Color used in drawing of the border.
     */
     protected Color borderDarkColor	= null;
    /**
     * Color used in drawing of the disabled text.
     */
     protected Color disabledTextColor	= null;
    /**
     * Cached value of the foreground color.  Used to determine if calculated colors need to be updated.
     */
     protected Color cachedForeground	= null;
    /**
     * Cached value of the background color.  Used to determine if calculated colors need to be updated.
     */
     protected Color cachedBackground	= null;

     private int TF_LEFT = 0;
     private int TF_RIGHT = 0;//-1;
     private int TF_TOP = 24;//30;
     private int TF_BOTTOM = -2;
     private int TF_BTN_HEIGHT = 20;     // sera surcharg


     private Font fReg;
     private Font fSel;

     private Component userPanel = null;

     private int iTabsPosition = TOP;
     private int iTabsStyle = ROUNDED;

     transient private int osAdjustment;

     private int lastWidth = -1;

     private Insets btpInsets;

     private Mouse	mouse	= null;
     private Action	action	= null;



    /**
     * Constructs a TabPanel with the desired tab position
     * and corner style.
     * @param tabsPostion a constant indicating TOP or BOTTOM tab location
     * @param tabsStyle a constant indicating ROUNDED etc... tabs
     */
     public TabPanel(int tabsPosition, int tabsStyle)
     {

	vLabels = new Vector();
	vEnabled = new Vector();
	vPolys = new Vector();
	btpInsets = new Insets(0,0,0,0);

	setTabsInfo(tabsPosition, tabsStyle);

	fReg = tabRegFont;  // DGS Allowed to change fonts
	fSel = tabSelFont;

    	if (System.getProperty("os.name").startsWith("S")) // SunOS, Solaris
	  osAdjustment = -1;
	else
          osAdjustment = 0;

	super.setLayout(null);

	// prepare left/right arrows
	dbLeft  = new DirButton(DirButton.LEFT);
	dbRight = new DirButton(DirButton.RIGHT);
	dbLeft.setShowFocus(false);
	dbRight.setShowFocus(false);

	super.add(dbLeft, -1);
	super.add(dbRight, -1);

	nullPoly = new Polygon();
	nullPoly.addPoint(0, 0);
	nullPoly.addPoint(1, 1);
	nullPoly.addPoint(0, 0);

	//Initilize the cached colors.
    	cachedForeground	= getForeground();
	cachedBackground	= getBackground();

        //Cause une exception
        //dbLeft.setBackground(getBackground());
        //dbRight.setBackground(getBackground());

        // Extension de TabPanel
	vPanels = new Vector();

     }

   /**
    * Constructs a TabPanel with tabs on top and rounded.
    */
   public TabPanel()
   {
	this(TOP, ROUNDED);
   }


   public void setBevel(int s) { bevel = s; }


   public void setBackground(Color b)
   {
     super.setBackground(b);
     if(dbLeft != null)  dbLeft.setBackground(b);
     if(dbRight != null) dbRight.setBackground(b);
   }

    /**
       Changement des fonts pour les tabs
    */

    Font tabRegFont = new Font("Dialog", Font.PLAIN, 14);
    Font tabSelFont = new Font("Dialog", Font.BOLD, 14);

    public void setTabFonts(Font reg, Font sel)
    {
      tabRegFont = reg;
      tabSelFont = sel;
    }


    /**
     * Sets the position of the all tabs to the top or bottom of this panel.
     * Note that if the tabs are on top they are rounded/cutted.
     * @param tabsPosition constant indicating TOP or BOTTOM
     * @exception PropertyVetoException
     * if the specified property value is unacceptable
     */
     public void setTabsPosition(int tabsPosition) // throws PropertyVetoException
     {
	if (iTabsPosition != tabsPosition)
	{
            setTabsInfo(tabsPosition, iTabsStyle);
	}
     }

    /**
     * Gets the current tabs position, TOP or BOTTOM.
     */
     public int getTabsPosition() { return iTabsPosition; }

    /**
     * Sets the style of the tabs
     */
     public void setTabsStyle(int tabsStyle)
     {
	if (iTabsStyle != tabsStyle)
	{
	  setTabsInfo(iTabsPosition, tabsStyle);
	}
     }

    /**
     * Gets the current tab style
     */
     public int getTabsStyle() {  return iTabsStyle; }

    /**
     * Sets the position and style of all the tabs.
     * Note that if the tabs are on top they are always rounded.
     * @param tabsPosition a constant indicating TOP or BOTTOM
     * @param tabsStyle a constant
     */
      public void setTabsInfo(int tabsPosition, int tabsStyle)
      {
	iTabsPosition = tabsPosition;
	iTabsStyle = tabsStyle;

	if (iTabsStyle == SLANTED && tabsPosition == BOTTOM)  TF_BTN_HEIGHT = 17;
	else TF_BTN_HEIGHT = 20;

	triggerRepaint();
       }

      public boolean setSuppressRepaints(boolean b)
      {
	boolean wasSuppressingRepaints = suppressRepaints;
	suppressRepaints = b;
	return wasSuppressingRepaints;
      }

	protected void triggerRepaint()
	{
	  if (!suppressRepaints)	repaint();
	}


	/**
	 * Labels and conditionally enables the tab at the specified index.
	 * @param sLabel the tab label
	 * @param bEnabled enable the tab or not
	 * @param index the zero-relative index of the tab
	 */
  public synchronized void setTab(String sLabel, boolean bEnabled, int index)
  {
	boolean wasSuppressingRepaints = setSuppressRepaints(true);

	try
	{
	  setLabel(sLabel, index);
	  setEnabled(bEnabled, index);
	}
	finally
	{
	  setSuppressRepaints(wasSuppressingRepaints);// Restitue la valeur
	}

	triggerRepaint();
  }

	/**
	 * Changes the label of the tab at the specified index.
	 * @param sLabel the new label for the specified tab
	 * @param index the zero-relative index of the tab
	 */
	public synchronized void setLabel(String sLabel, int index)
	{
		try
		{
			vLabels.setElementAt(sLabel, index);
			triggerRepaint();
		}
		catch (ArrayIndexOutOfBoundsException e) {}
	}

	/**
	 * Gets the label of the tab at the specified index.
	 * @param index the zero-relative index of the tab
	 * @return the tab label
	 * @see #setLabel
	 */
	public synchronized String getLabel(int index)
	{
		try
		{
			return (String)vLabels.elementAt(index);
		}
		catch (ArrayIndexOutOfBoundsException e) {}
		return "";
	}

    /**
     * Sets the tab labels associated with the panel positions.
     * Note that the panels do not need to have been added yet for
     * this method to work.
     * @param sLabels an array of tab labels for the panel positions
     */
	public void setPanelLabels(String[] sLabels)
	{
	    labels=sLabels;
	    updatePanelLabels();
	}

    /**
     * Gets the current tab labels associated with the panel positions.
     * @return an array of tab labels for the panel positions
     */
	public String[] getPanelLabels()
	{
	    return labels;
	}



	/**
	 * Conditionally enables the tab at the specified index.
	 * The currently active tab cannot be disabled.
	 * @param bEnable true to enable, false to disable
	 * @param index the zero-relative index of the tab
	 */
	public synchronized void setEnabled(boolean bEnabled, int index)
	{
		if ((index < 0) || (index >= vLabels.size()))	return;
		if (index == curIndex && !bEnabled)			return;

		try
		{
			Boolean newValue = new Boolean(bEnabled);
			vEnabled.setElementAt(newValue, index);
			triggerRepaint();
		}
		catch (ArrayIndexOutOfBoundsException e) {}
	}

	/**
	 * Determines whether or not the tab at the index is enabled.
	 * @param index the zero-relative index of the tab
	 * @return true if the tab at the index is enabled
	 */
	public boolean isEnabled(int index)
	{
		try
		{
			Boolean bool = (Boolean) vEnabled.elementAt(index);
			if (bool.booleanValue()) return true;
		}
		catch (ArrayIndexOutOfBoundsException e) {}

		return false;
	}


	/**
	 * Gets the index for a specific panel.
	 * @param panel the panel to get the index of
	 */
	public synchronized int getPanelIndex(Component panel)
	{
		return vPanels.indexOf(panel);
	}



	/**
	 * Selects the tab at the given index, showing it
	 * and its associated panel.
	 * The panel is activated, ready for user input.
	 * The tab position must be enabled.
	 * @param index zero-relative index of the tab to select
	 */

	public void setPanel(int index)
	{
	   if(index == curIndex) return;
	   select(index);
	}

   public synchronized void select(int index)
   {
     if (isEnabled(index))
      {
	try
	 {
		Component p = (Component) vPanels.elementAt(index);
		setCurrentTab(index);
		showPanel(p);
	  }
	 catch (ArrayIndexOutOfBoundsException e) {}
       }
  }


	/**
	 * Shows the tab at the specified index, if enabled
	 */

	// positionne l'index de tab

	public int previousIndex = 0;

	public void setCurrentTab(int index)
	{
	if (index == curIndex)	return;

	if ( isEnabled(index) )
	{
//	  boolean wasSuppressingRepaints = setSuppressRepaints(true);

	  try
	  {
	     previousIndex = curIndex;
	     curIndex = index;
//	     invalidate();
//	     validate();
	   }
	  finally
	   {
//	      setSuppressRepaints(wasSuppressingRepaints);
	   }

//	       triggerRepaint();
	  }
     }

	/**
     * Adds the panel to the base panel and shows it.
     * Hides all other (previous) panels instead of removing them.
     * @param p the Panel to add and show
     */
    public void showPanel(Component p)
    {
	if (userPanel != null)	userPanel.setVisible(false);

	userPanel = p;
	if (userPanel != null)
	{
		Component[] comps = getComponents();
		int l = comps.length;
		int x;
		// recherche le panel en argument
		for (x = 0; x < l; x++)
		{
			if (comps[x] == userPanel) break;
		}
		if (x == l) super.add(userPanel, -1);

		userPanel.setVisible(true);
		userPanel.requestFocus();
		validate();
		triggerRepaint();
	    }
    }


    /**
       Restore previous tab
    */
    public void backTab() { setCurrentTab(previousIndex); }

	/**
	 * Determine the currently shown tab.
	 * @return zero-relative index of the currently shown tab
	 */
    public int getCurrentTab() { return curIndex; }
    public int getTab() { return curIndex; }


	/**
	 * Appends a new tab (no panel) and sets whether it is enabled.
	 * @param sLabel the tab label
	 * @param bEnabled enable the tab or not
	 * @return the zero-relative index of the newly added tab
	 * @see #setTab
	 * @see #setLabel
	 */
	public int addTab(String sLabel, boolean bEnabled)
	{
		return addTab(sLabel, bEnabled, -1);
	}

	/**
	 * Adds a new tab at the specified position and sets whether it is enabled.
	 * @param sLabel the tab label
	 * @param bEnabled enable the tab or not
	 * @param pos the zero-relative index for the new tab
	 * @return the zero-relative index of the newly added tab
	 * @see #setTab
	 * @see #setLabel
	 */
	public int addTab(String sLabel, boolean bEnabled, int pos)
	{
		int index;

		if (pos == -1) {
		vLabels.addElement(sLabel);
		vEnabled.addElement(new Boolean(bEnabled));
		index = vLabels.size() - 1;
		}
		else {
		vLabels.insertElementAt(sLabel, pos);
		vEnabled.insertElementAt(new Boolean(bEnabled), pos);
		index = pos;
		}
		if (curIndex == -1 && bEnabled)
		{
 		  setCurrentTab(index);
		}
		return (index);
	}


	/**
	 * Inserts a tab before the tab at the specified index.
	 * @param sLabel label of the new tab to insert
	 * @param bEnabled enable the tab or not
	 * @param index the zero-relative index at which the tab will be inserted.
	 */
    public synchronized void insertTab(String sLabel, boolean bEnabled, int index)
    {
        if (index == curIndex && !bEnabled)           return;

        try
        {
            vLabels.insertElementAt(sLabel, index);
            vEnabled.insertElementAt(new Boolean(bEnabled), index);
            triggerRepaint();
        }
        catch (ArrayIndexOutOfBoundsException e) {}
    }


    /**
     * Appends a new tab and associated panel, which will be shown when the
     * tab is selected. The tab/panel is added after the last existing
     * panel.
	 * @param sLabel the tab label
	 * @param bEnabled enable the tab or not
     * @param panel the panel to associate with the tab
	 * @return the zero-relative index of the newly added tab panel
     */

	public int addTabPanel(String sLabel, boolean bEnabled, Component panel)
	{
		return addTabPanel(sLabel, bEnabled, panel, -1);
	}

    /**
     * Adds a new tab and associated panel.
     * The tab/panel is added at the specified index.
	 * @param sLabel the tab label
	 * @param bEnabled enable the tab or not
     * @param panel the panel to associate with the tab
     * @param pos the zero-relative index of the new tab panel
	 * @return the zero-relative index of the newly added tab panel
     */
	public int addTabPanel(String sLabel, boolean bEnabled, Component panel, int pos)
	{
		if (pos == -1)
		    vPanels.addElement(panel);
		else
		    vPanels.insertElementAt(panel, pos);

		return addTab(sLabel, bEnabled, pos);
	}



	/**
	 * Inserts a tab panel based on index.
	 * @param sLabel Label of the new tab to insert
	 * @param bEnabled If the new tab is enabled or not
	 * @param panel The panel to insert
	 * @param index zero-relative index at which the tab panel will be inserted.
	 */
	public synchronized void insertTabPanel(String sLabel, boolean bEnabled, Component panel, int index)
    {
        if ((index < 0) || (index >= vPanels.size()))  return;
        if (index == getCurrentTab() && !bEnabled)     return;

        try
        {
            vPanels.insertElementAt(panel, index);
            insertTab(sLabel, bEnabled, index);
        }
        catch (ArrayIndexOutOfBoundsException e) {}
    }



    /**
        Autre nom
     */
	public synchronized Component add(String name, Component comp)
	 {
	   addTabPanel(name, true, comp);
	   return comp;
	 }


    /**
     * Adds a component to the end of this container.
     * @param comp the component to add
     * @return the added component
     */

	public Component add(Component comp)
	 {
	   addTabPanel(createDefaultLabel(vPanels.size()), true, comp);
	   return comp;
	 }



    // Les mthodes suivantes pour ajouter un panneau
    // aprs affichage

    /**
     * Adds a component to the end of this container.
     * It is overridden so that it only appends to the TabPanel.
     * If the tab label for this position has not been set it is given
     * the name "tab - #", where # is the zero-relative index of its
     * position.
     *
     * @param comp the component to add
     * @param pos the zero-relative index at which to add the component or -1
     * for end (IGNORED)
     * @return the added component
     */
	public synchronized Component add(Component comp, int pos)
	{
	  add(createDefaultLabel(vPanels.size()), comp, pos);
	  return comp;
	}

	public synchronized Component add(String lab, Component comp, int pos)
	{
	 boolean wasSuppressingRepaints = setSuppressRepaints(true);

	 try
	  {
		int newIndex = addTabPanel(lab, true, comp, pos);
 	    // If this is the panel that we've set to be the default, or we're designing,
		// go ahead and switch to the new panel.
		if (newIndex == curIndex) select(newIndex);
		updatePanelLabels();
 	  }
	 finally
	  {
		setSuppressRepaints(wasSuppressingRepaints);
	  }

	 triggerRepaint();
	 return comp;
	}


    /**
     * Sets the tab label  associated with the current panels at the given index.
     * @param String newLabel the label to use for the tab at the given index.
     * @param int labelIndex an index in the array of tab labels for the current panels.
     */
	public void setPanelLabel(String newLabel, int labelIndex)
	{
		if(labelIndex >= 0 && labelIndex < labels.length && labels[labelIndex] != newLabel)
		{
		    labels[labelIndex] = newLabel;
		    updatePanelLabels();
		}

	}

    /**
     * Gets the current tab label associated with the current panels at the given index.
     * @param int labelIndex an index in the array of tab labels for the current panels.
     * @return String the label of the panel at the given index. Null if the index is out of range.
     */
	public String getPanelLabel(int labelIndex)
	{
		if(labelIndex >= 0 && labelIndex < labels.length)
		{
	    	return labels[labelIndex];
	    }
	    else
	    	return null;
	}



  /**
     * Replaces a tab and its associated panel at the index specified.
     * If it is desired to only change the label, use the base class
     * TabPanel's method setTab(String sLabel, boolean bEnabled, int index).
	 * @param sLabel the new tab label
	 * @param bEnabled enable the tab or not
     * @param panel the new panel
     * @param index the zero-relative index of the tab to change
     */
	public synchronized void setTabPanel(String sLabel, boolean bEnabled, Component panel, int index)
	{
		if ((index < 0) || (index >= vPanels.size())) return;
		if (index == getCurrentTab() && !bEnabled)    return;

		try
		{
			vPanels.setElementAt(panel, index);
			setTab(sLabel, bEnabled, index);

		}
		catch (ArrayIndexOutOfBoundsException e) {}

	}

	/**
	 * Gets the panel for the tab at the given index.
	 * @param index zero-relative index of the tab
	 * @return returns the Panel associated with the tab
	 * @see #setTabPanel
	 */
	public synchronized Component getTabPanel(int index)
	{
		if ((index < 0) || (index >= vPanels.size()))
			return null;

		Component p = null;
		try
		{
			p =  (Component)vPanels.elementAt(index);
		}
		catch (ArrayIndexOutOfBoundsException e) {}

		return p;
	}



    /**
	 * Removes the specified component from this container.
	 *
	 * @param comp the component to remove
	 */
	public synchronized void remove(Component comp)
	{
		int i = getPanelIndex(comp);

		if (countTabs()==1)	removeAllTabPanels();
		else
		{
			if (i==0)	select(1);
			else		select(i-1);
			removeTabPanel(i);
		}
	}


	/**
	 * Removes the tab at the specified index.
	 * The currently shown tab cannot be removed.
	 * @param index the zero-relative index of the tab to remove
	 */
	public void removeTab(int index)
	{
		if (index == curIndex) return;

		try
		{
			vLabels.removeElementAt(index);
			vEnabled.removeElementAt(index);
			triggerRepaint();
		}
		catch (ArrayIndexOutOfBoundsException e) {}
	}

	/**
	 * Removes all tabs.
	 */
	public void removeAllTabs()
	{
		vLabels = new Vector();
		vEnabled = new Vector();
		vPolys = new Vector();
		curIndex = -1;
		firstVisibleTab = 0;
		lastWidth = -1;
		removeAll();
		triggerRepaint();
	}



	/**
	 * Removes a tab and its associated panel at the given index.
	 * The currently active tab cannot be removed.
	 * @param index zero-relative index of the tab
	 */
	public synchronized void removeTabPanel(int index)
	{
		if ((index < 0) || (index >= vPanels.size()) || index == curIndex)
			return;

		try
		{
			Component p = (Component) vPanels.elementAt(index);
			super.remove(p);
			vPanels.removeElementAt(index);
		}
		catch (ArrayIndexOutOfBoundsException e) {}

		removeTab(index);
	}

	/**
	 * Removes all tabs and their associated panels, clearing the TabPanel entirely.
	 */
	public synchronized void removeAllTabPanels()
	{
		vPanels = new Vector();
		curIndex = -1;
		removeAllTabs();
	}

    /**
     * This routine re-sets all the tab labels using the latest string array
     * provided in setPanelLabels().
     * It is not typically called directly.
     * @see #setPanelLabels
     */
	public void updatePanelLabels()
	{
	    try
	    {
	    	for (int i = 0; i < vPanels.size(); i++)
		    {
				String newlabel;
				if (labels!=null)
				{
					try
					{
			    		newlabel=labels[i];
					}
					catch (ArrayIndexOutOfBoundsException e)
					{
			    		newlabel=createDefaultLabel(i);
					}
				}
				else
			    	newlabel=createDefaultLabel(i);
				setLabel(newlabel,i);
		    }
	    }
	    catch(Throwable thr) {}
	}


    /** DGS
       Set borders for component futurly added inside
    */
    int marginl = 3;
    int margint = 3;
    int marginr = 6;
    int marginb = 5;

    public void setMargins(int l, int t, int r, int b)
    {
      marginl = l;
      margint = t;
      marginr = r;
      marginb = b;
    }

	/**
     * Handles the laying out of components within this component.
     * This is a standard Java AWT method which gets called by the AWT
     * when this component is validated with the validate() method.
     *
     * @see java.awt.Container#validate
	 */
	public void doLayout()
	{
		Rectangle r = getBounds();

		int width = r.width - TF_LEFT + TF_RIGHT;
		if (width < 0) return;

		int height = r.height - TF_TOP + TF_BOTTOM;
		if (height < 0)	return;

		int col = TF_LEFT;
		int row = 0;

		if (iTabsPosition == TOP) row = TF_TOP;
		else	row = TF_TOP - TF_BTN_HEIGHT;

		if (userPanel != null)
		{
			userPanel.setBounds(col + marginl, row + margint, width-marginr, height-marginb);
			userPanel.invalidate();
			userPanel.validate();
            if (userPanel instanceof Canvas || userPanel instanceof Panel)
            {
			    userPanel.repaint();
		    }
		    else
		    {
		        triggerRepaint();
        	}

		}
	}



    /**
        DGS
    */

    void drawUnderlined(Graphics g, String s, int x, int y)
    {
     if(s.charAt(0) == '+')
     {
       g.drawString("_", x, y);
       g.drawString(s.substring(1, s.length()), x, y);
     }
     else
       g.drawString(s, x, y);
    }

    void drawNoUnderlined(Graphics g, String s, int x, int y)
    {
     if(s.charAt(0) == '+')
     {
       g.drawString(s.substring(1, s.length()), x, y);
     }
     else
       g.drawString(s, x, y);
    }

	/**
	 * Paints this component using the given graphics context.
     * This is a standard Java AWT method which typically gets called
     * by the AWT to handle painting this component. It paints this component
     * using the given graphics context. The graphics context clipping region
     * is set to the bounding rectangle of this component and its [0,0]
     * coordinate is this component's top-left corner.
     *
     * @param g the graphics context used for painting
     * @see java.awt.Component#repaint
     * @see java.awt.Component#update
	 */
	public synchronized void paint(Graphics g)
	{
		Rectangle r = getBounds();

		//Make sure cached colors are correct.
		Color curForeground = getForeground();
		Color curBackground = getBackground();

		if (!objectsEqual(curForeground, cachedForeground))
		{
			cachedForeground = curForeground;
			calculateDisabledTextColor(curForeground);
		}
		if (!objectsEqual(curBackground, cachedBackground))
		{
			cachedBackground = curBackground;
			calculateBorderColors(curBackground);
		}


		// --------------------------------------------------------------------------------
		// paint the box
		// --------------------------------------------------------------------------------

		int width = r.width - TF_LEFT + TF_RIGHT;
		if (width < 0)	return;

		int height = r.height - TF_TOP + TF_BOTTOM;
		if (height < 0)	return;

		if (r.width > lastWidth) firstVisibleTab = 0;
		lastWidth = r.width;

		int col = TF_LEFT;
		int row;

		Color c = g.getColor();
		g.setColor(curBackground);
		g.fillRect(0, 0, r.width, r.height);

		if (iTabsPosition == TOP)
		   row = TF_TOP;
		else
		   row = TF_TOP - TF_BTN_HEIGHT; // Plus haut


                int strOffset = 8;   // dcalage de la chaine dans le tab
                if(iTabsStyle == SLANTED) strOffset = 14;

		// --------------------------------------------------------------------------------
		// draw border
		// --------------------------------------------------------------------------------

		g.setColor(borderLightColor);
		g.drawLine(col, row, (col + width - 1), row);

		if(bevel == RAISED)
		{
		  g.drawLine(col, row, col, (row + height - 1));

		  g.setColor(borderDarkColor);
		  g.drawLine((col + 2), (row + height - 2), (col + width - 2), (row + height - 2));
		  g.drawLine((col + width - 2), (row + 2), (col + width - 2), (row + height - 2));

		  g.setColor(borderDarkerColor);
		  g.drawLine((col + 1), (row + height - 1), (col + width - 1), (row + height - 1));
		  g.drawLine((col + width - 1), (row + 1), (col + width - 1), (row + height - 1));
                }

		// --------------------------------------------------------------------------------
		// paint the tabs, and record areas
		// --------------------------------------------------------------------------------
		int x1;
		int x2 = TF_LEFT + 8;           // TF_LEFT vaut 0
		int y1;                         // Le haut des tab (le bas si invers)
		int y2;                         // la base des tab (ev. invers)
		int x3 = 0;
		int x4 = TF_LEFT;
		int offset = 0;

		int sze = vLabels.size();
		String sLabel;
		vPolys.removeAllElements();

		Font f = g.getFont();
		FontMetrics fm = getFontMetrics(fReg);
		FontMetrics fms = getFontMetrics(fSel);
		int labelWidth = 0;
		Polygon p;

		int w;
		// make sure there is a polygon for each tab
		for (w = 0; w < firstVisibleTab; w++)
		{
			vPolys.addElement(nullPoly);
		}
		if (w > 0)	x4 += 2;

        // Pour chaque tab visible
		for (; w < sze; w++)
		{
			p = new Polygon();
			try
			{
				sLabel = (String) vLabels.elementAt(w);
				String refLabel;
				if(sLabel.charAt(0) == '+')
				 refLabel = sLabel.substring(1, sLabel.length());
				else
				 refLabel = sLabel;
				if (w == curIndex)
					labelWidth = fms.stringWidth(refLabel);
				else
					labelWidth = fm.stringWidth(refLabel);

				if (iTabsPosition == TOP)
				{
					y1 = TF_TOP - TF_BTN_HEIGHT;
					y2 = TF_TOP - 1;
				}
				else
				{
					y1 = r.height + TF_BOTTOM + 1;// modifi
					y2 = r.height + TF_BOTTOM - TF_BTN_HEIGHT;
				}

				if (iTabsStyle == ROUNDED)
				{
					x1 = x4 + 2;    // TF_LEFT + 2 au dpart, puis incrment par tab
					x2 = x1 + labelWidth + 13;
				}
				else
				if(iTabsStyle == CUTTED)
				{
					x1 = x4 + 2;
					x2 = x1 + labelWidth + 13;
				    offset = 4;
				}
				else // SLANTED
				{
					x1 = x2 - 7;
					x2 = x1 + labelWidth + 28;
				}

				// check to see if this tab would draw too far
				if ( (x2 + 36 - TF_RIGHT) > r.width )	break;

				// draw the outside edge of the tab
				if (iTabsPosition == TOP)
				{
					if (iTabsStyle == SLANTED)
					{
					    g.setColor(borderLightColor);
						g.drawLine(x1 + 9, y1, x2 - 9, y1);  // top

						// left / slanted line
						if (w == 0 || w == curIndex)
						{
							g.drawLine(x1, y2, x1 + 9, y1);  // ligne gauche entire
							p.addPoint(x1, y2);  // sud-ouest
						}
						else
						{
							g.drawLine(x1 + 4, y1 + 9, x1 + 9, y1); // moiti sup.
							p.addPoint(x1 + 9, y2);      // nord-ouest
							p.addPoint(x1 + 4, y1 + 9);  // point intermdiaire gauche
						}
						p.addPoint(x1 + 9, y1); // nord-ouest, dbut de ligne sup
						p.addPoint(x2 - 9, y1); // nord-est, fin de ligne sup

                                                g.setColor(borderDarkColor);
						if ((w+1) == curIndex)  // Si suivant slectionn
						{
						  g.drawLine(x2 - 9, y1 , x2 - 5, y1 + 9); // moiti bord droit
						  p.addPoint(x2 - 9, y1); // nord-est
						  p.addPoint(x2 - 5, y2); // sud-est trci
						}
						else
						{
							g.drawLine(x2, y2, x2 - 9, y1);  // bord droit entier
							p.addPoint(x2, y2);  // sud-est
						}

						if (w == 1 || w == curIndex)
							p.addPoint(x1, y2);
						else
							p.addPoint(x1 + 9, y2);

						g.setColor(borderDarkerColor); // accentue le relief
						if ((w+1) == curIndex)
						  g.drawLine(x2 - 8, y1 + 1 , x2 - 4, y1 + 9);
						else
						  g.drawLine(x2 + 1, y2, x2 - 8, y1 + 1);

				    } // slanted
                 else // non slanted
                 {
					// if current tab, it extends further
					if (w == curIndex)
					{
						y1 -= 3;
						x1 -= 2;
					}

					// Bord clair top
					g.setColor(borderLightColor);

					if (curIndex == (w + 1))
						g.drawLine(x1+2 + offset, y1, x2-2, y1);
					else
						g.drawLine(x1+2 + offset, y1, x2, y1);

					// draw the border between tabs if not covered by the current one
					if (curIndex != (w - 1))
					{
						g.drawLine(x1, y1+2 + offset, x1, y2);
						x3 = x1;
					}
					else
						x3 = x1 + 1;

					g.drawLine(x1+1, y1+1 + offset, x1+1 + offset, y1+1);  // point du coin gauche

					if (curIndex != (w + 1))
					{
						g.setColor(borderDarkColor);
						g.drawLine(x2, y1, x2, y2);       // right
						g.setColor(borderDarkerColor);
						g.drawLine(x2+1, y1+2, x2+1, y2); // right
						x4 = x2;
					}
					else
						x4 = x2 - 1;
				  } // fin non slanted

				} // fin top

				// Sinon BOTTOM
				else
				{
					if (iTabsStyle == SLANTED)
					{
						g.setColor(borderDarkColor);
						g.drawLine(x1+9, y1, x2-9, y1);

						g.setColor(borderDarkerColor);
						// left \ slanted line
						if (w == 0 || w == curIndex)
						{
							g.drawLine(x1, y2, x1+9, y1);
							p.addPoint(x1, y2);
						}
						else
						{
							g.drawLine(x1+4, y1-9, x1+9, y1);
							p.addPoint(x1+9, y2);
							p.addPoint(x1+4, y1-9);
						}
						p.addPoint(x1+9, y1);
						p.addPoint(x2-9, y1);

						if ((w+1) == curIndex)
						{
							g.drawLine(x2-5, y1-9, x2-9, y1);
							p.addPoint(x2-5, y1);
							p.addPoint(x2-9, y2);
						}
						else
						{
							g.drawLine(x2, y2, x2-9, y1);
							p.addPoint(x2, y2);
						}

						if (w == 1 || w == curIndex)
							p.addPoint(x1, y2);
						else
							p.addPoint(x1+9, y2);
					}
					else                // ROUNDED / CUTTED
					{
						// if current tab, it extends further
						if (w == curIndex)
						{
							y1 += 3;
							x1 -= 2;
						}
						g.setColor(borderLightColor);
						if (curIndex == (w + 1))
							g.drawLine(x1+2, y1, x2-2, y1);
						else
							g.drawLine(x1+2, y1, x2, y1);

						// draw the border between tabs if not covered by the current one
						if (curIndex != (w - 1))
						{
							g.drawLine(x1, y1-2, x1, y2);
							x3 = x1;
						}
						else
							x3 = x1 + 1;

						g.drawLine(x1+1, y1-1, x1+1, y1-1);

						if (curIndex != (w + 1))
						{
							g.setColor(borderDarkColor);
							g.drawLine(x2, y1, x2, y2);
							g.setColor(borderDarkerColor);
							g.drawLine(x2+1, y1-2, x2+1, y2);
							x4 = x2;
						}
						else
							x4 = x2 - 1;
					} // fin rounded/cutted
				} // fin bottom

				// efface bord qui spare tab du panel, si slect
				boolean current = false;
				if (w == curIndex)
				{
				    current = true;
					if (iTabsPosition == TOP) y2++;
					                     else y2--;
					g.setColor(curBackground);
					g.drawLine(x1+1, y2, x2, y2);
					if (iTabsPosition == BOTTOM)
						g.drawLine(x1+1, y2-1, x2, y2-1);

					g.setFont(fSel);
				}
				else
					g.setFont(fReg);

				if (iTabsStyle == ROUNDED || iTabsStyle == CUTTED)
				{
					p.addPoint(x3, y2);
					p.addPoint(x4, y2);
					p.addPoint(x4, y1);
					p.addPoint(x3, y1);
					p.addPoint(x3, y2);
				}
				vPolys.addElement(p);

				Boolean bool = (Boolean) vEnabled.elementAt(w);
				if (bool.booleanValue())
					g.setColor(curForeground);
				else
					g.setColor(disabledTextColor);

				if (iTabsPosition == TOP)
				{
				 if(current)
				   drawNoUnderlined(g, sLabel, x1 + strOffset, y1 + 15 + osAdjustment);
				 else
				   drawUnderlined(g, sLabel, x1 + strOffset, y1 + 15 + osAdjustment);
				}
				else
				{
					if (iTabsStyle == ROUNDED || iTabsStyle == CUTTED)
						g.drawString(sLabel, x1 + strOffset , y1 - 6 + osAdjustment);
					else
						g.drawString(sLabel, x1+ strOffset, y1 - 4 + osAdjustment);
				}
			}
			catch (ArrayIndexOutOfBoundsException e) {}
		} // fin loop

		// do I need to show arrows because there are too many tabs???
		if ( (firstVisibleTab > 0) || (w < sze) )
		{
			dbLeft.setVisible(true);
			dbRight.setVisible(true);
			if (firstVisibleTab > 0)
				dbLeft.setEnabled(true);
			else
				dbLeft.setEnabled(false);

			if (w < sze)
				dbRight.setEnabled(true);
			else
				dbRight.setEnabled(false);

			if (iTabsPosition == TOP)
			{
				dbLeft.setBounds(r.width-33+TF_RIGHT, TF_TOP - 16, 16, 15);
				dbRight.setBounds(r.width-16+TF_RIGHT, TF_TOP - 16, 16, 15);
			}
			else
			{
				dbLeft.setBounds(r.width-33+TF_RIGHT, r.height + TF_BOTTOM - TF_BTN_HEIGHT, 16, 15);
				dbRight.setBounds(r.width-16+TF_RIGHT, r.height + TF_BOTTOM - TF_BTN_HEIGHT, 16, 15);
			}
		}
		else
		{
			dbLeft.setVisible(false);
			dbRight.setVisible(false);
		}

		// make sure there is a polygon for each tab
		for (; w < sze; w++)
		{
			vPolys.addElement(nullPoly);
		}

		g.setFont(f);
		g.setColor(c);
	}



    /**
	 * Removes all the components from this container.
	 * This is a standard Java AWT method which gets called to remove all
	 * the components from a container. When this happens each component's
	 * removeNotify() will also get called to indicate component removal.
	 *
     * @see #remove
     * @see #addTab
	 */
    public synchronized void removeAll()
    {
    	super.removeAll();
		super.add(dbLeft, -1);
		super.add(dbRight, -1);
		userPanel = null;
    }

	/**
	 * Takes no action.
	 * Standard Java AWT method
	 * Since layout managers CANNOT BE USED with this container the standard
	 * setLayout has been OVERRIDDEN for this container and does nothing.
	 **/
    public void setLayout(LayoutManager mgr) {}

    /**
     * Returns the amount of space used by the current border.
     * @return the current border insets
     */
	public Insets getInsets()
	{
		btpInsets = super.getInsets();
		btpInsets.left += (TF_LEFT + 3);
		btpInsets.right += (6 - TF_RIGHT);

		if (iTabsPosition == TOP)
		{
			btpInsets.top += (TF_TOP + 3);
			btpInsets.bottom += (5 - TF_BOTTOM);
		}
		else
		{
			btpInsets.top += TF_TOP - TF_BTN_HEIGHT + 3;
			btpInsets.bottom += (TF_BTN_HEIGHT + 5 - TF_BOTTOM);
		}

		return btpInsets;
    }



	/**
	 * Tells this component that it has been added to a container.
	 * Standard Java AWT method.
	 * It has been overridden here to hook-up event listeners.
	 *
	 * @see #removeNotify
	 */
	public synchronized void addNotify()
	{
		super.addNotify();

		//Hook up listeners
		if (mouse == null)
		{
			mouse = new Mouse();
			addMouseListener(mouse);
		}
		if (action == null)
		{
			action = new Action();
			dbLeft.addActionListener(action);
			dbRight.addActionListener(action);
		}
	}

	/**
	 * Tells this component that it is being removed from a container.
	 * Standard Java AWT method.
	 * It has been overridden here to unhook event listeners.
	 */
	public synchronized void removeNotify()
	{
		//Unhook listeners
		if (mouse != null)
		{
			removeMouseListener(mouse);
			mouse = null;
		}
		if (action != null)
		{
			dbLeft.removeActionListener(action);
			dbRight.removeActionListener(action);
			action = null;
		}

		super.removeNotify();
	}


	/**
	 * This is the Mouse Event handling innerclass.
	 */
	class Mouse extends java.awt.event.MouseAdapter implements java.io.Serializable
	{
		public void mousePressed(MouseEvent e)
		{
			int sizeR = vPolys.size();
			Polygon p;
			for (int x = 0; x < sizeR; x++)
			{
				try
				{
					p = (Polygon) vPolys.elementAt(x);
					if ( (p != nullPoly) && p.contains(e.getX(), e.getY()) )
					{
					  // remplace select car on ne doit pas rafficher le mme
                      select(x);
					}
				}
				catch (ArrayIndexOutOfBoundsException exc){}
			}
		}
	}


	/**
	 * This is the Action Event handling innerclass.
	 */
	class Action implements java.awt.event.ActionListener, java.io.Serializable
	{
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() == dbLeft)
			{
				if (--firstVisibleTab < 0)
					firstVisibleTab = 0;
				else
					triggerRepaint();
			}
			else if (e.getSource() == dbRight)
			{
				int sze = vLabels.size();
				if (++firstVisibleTab == sze)
					firstVisibleTab--;
				else
					triggerRepaint();
			}
		}
	}

	/**
	 * Used to calculate the border colors from the background color.
	 * @see #paint
	 */
	protected void calculateBorderColors(Color c)
	{
		borderLightColor	= ColorUtils.calculateHilightColor(c);
		borderDarkColor		= ColorUtils.calculateShadowColor(c);
		borderDarkerColor	= ColorUtils.darken(borderDarkColor, (double) 1 / 5);
	}

	/**
	 * Used to calculate the disabled text color from the foreground color.
	 * @see #paint
	 */
	protected void calculateDisabledTextColor(Color c)
	{
		try
		{
			disabledTextColor = ColorUtils.fade(c, Color.lightGray, (double) 1 / 2);
		}
		catch (IllegalArgumentException exc) {}
	}


 /**  Compare deux objets  */
  public static boolean objectsEqual(Object objectA, Object objectB)
   {
		if (objectA == null) return (objectB == null);
		return objectA.equals(objectB);
   }





	/**
	 * Gets the number of tab panels in the TabPanel.
	 * @return the number of tab panels currently in the TabPanel
	 */
	public int countTabs()
	{
		return vPanels.size();
	}

   /**
     * Returns the recommended dimensions to properly display this component.
     * Standard Java AWT method.
     * The returned size is large enough to display the biggest tab panel
     * at its preferred size.
     */

    public Dimension getPreferredSize()
    {
    	Dimension p = getSize();
    	Dimension m = getMinimumSize();
    	return new Dimension(Math.max(p.width, m.width), Math.max(p.height, m.height));
    }

    /**
     * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     * <p>
     */
    public Dimension getMinimumSize()
    {
        return new Dimension(20, 40);
    }


	/**
	 * Creates a default name for a tab.
	 * @param int i Number to display as part of the tab's label.
	 * @return String The name to use for the new label.
	 * @see add(Component comp, int pos)
	 * @see updatePanelLabels()
	 */
	private String createDefaultLabel(int i)
	{
	    return ("Tab " + String.valueOf(i));
	}


}

