/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import common.*;

public class VectorBoolean
{
    /**
     * The array buffer into which the components of the VectorBoolean are
     * stored. The capacity of the VectorBoolean is the length of this array buffer.
     */

    public boolean elementData[];

    /** The number of valid components in the VectorBoolean  */
    public int elementCount;

    /**
     * The amount by which the capacity of the VectorBoolean is automatically
     * incremented when its size becomes greater than its capacity. If
     * the capacity is <code>0</code>, the capacity of the VectorBoolean is
     * doubled each time it needs to grow.
     */
    protected int capacityIncrement;
	protected int initialCapacity;    


    /**
     * Constructs an empty VectorBoolean with the specified initial capacity and
     * capacity increment.
     */

    public VectorBoolean(int initialCapacity, int capacityIncrement)
    {
	 super();
	 this.elementData = new boolean[initialCapacity];
	 this.capacityIncrement = capacityIncrement;
	 this.initialCapacity = initialCapacity;
    		//{{INIT_CONTROLS
		//}}
}

    /**
     * Constructs an empty VectorBoolean with the specified initial capacity.
     */

    public VectorBoolean(int initialCapacity) { this(initialCapacity, 1); }

    /** Constructs an empty VectorBoolean   */

    public VectorBoolean() { this(16); }


    /** DGS remet  zro */
    public void clear()
    {
	 this.elementData = new boolean[initialCapacity];
     elementCount = 0;
    }

    /**
     * Copies the components of this VectorBoolean into the specified array.
     * The array must be big enough to hold all the objects in this  VectorBoolean.
     */

    public final synchronized void copyInto(boolean anArray[])
    {
	 int i = elementCount;
	 
	 while (i-- > 0) {  anArray[i] = elementData[i]; }
    }

    /**
     * Trims the capacity of this VectorBoolean to be the VectorBoolean's current
     * size. An application can use this operation to minimize the
     * storage of a VectorBoolean.
     */

    public final synchronized void trimToSize()
    {
	 int oldCapacity = elementData.length;
	 if (elementCount < oldCapacity)
	  {
	    boolean oldData[] = elementData;
	    elementData = new boolean[elementCount];
	    copy(oldData, elementData, elementCount);
	  }
    }

    /**
     * Increases the capacity of this VectorBoolean, if necessary, to ensure
     * that it can hold at least the number of components specified by
     * the minimum capacity argument.
     */

    public final synchronized void ensureCapacity(int minCapacity)
    {
	 int oldCapacity = elementData.length;
	 
	 if (minCapacity > oldCapacity)
	  {
	    boolean oldData[] = elementData;
	    int newCapacity = (capacityIncrement > 0) ?
		(oldCapacity + capacityIncrement) : (oldCapacity * 2);

        if (newCapacity < minCapacity) { newCapacity = minCapacity; }

	    elementData = new boolean[newCapacity];
	    
        copy(oldData, elementData, elementCount);
	  }
    }

    /**
     * Sets the size of this VectorBoolean. If the new size is greater than the
     * current size, new <code>null</code> items are added to the end of
     * the VectorBoolean. If the new size is less than the current size, all
     * components at index <code>newSize</code> and greater are discarded.
     */

    public final synchronized void setSize(int newSize)
    {
	 if (newSize > elementCount) { ensureCapacity(newSize); }
	 else
	  {
	    for (int i = newSize ; i < elementCount ; i++)
	    {
		  elementData[i] = false;
	    }
	  }
	 elementCount = newSize;
    }

    /** Returns the current capacity of this VectorBoolean */

    public final int capacity() { return elementData.length; }

    /** Returns the number of components in this VectorBoolean */

    public final int size() { return elementCount; }

    /** Tests if this VectorBoolean has no components */

    public final boolean isEmpty() { return elementCount == 0; }


    /** Returns the component at the specified index  */

    public final synchronized boolean elementAt(int index)
    {
	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

  	  return elementData[index];
    }


    /**
     * Sets the component at the specified <code>index</code> of this
     * VectorBoolean to be the specified object. The previous component at that
     * position is discarded.
     */
    public final synchronized void setElementAt(boolean obj, int index)
    {
 	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

	  elementData[index] = obj;
    }

    /**
     * Deletes the component at the specified index. Each component in
     * this VectorBoolean with an index greater or equal to the specified
     * <code>index</code> is shifted downward to have an index one
     * smaller than the value it had previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than the current size of the VectorBoolean.
     */

    public final synchronized void removeElementAt(int index)
    {
	 if (index >= elementCount)
	  throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);
	 else
	 if (index < 0)
      throw new ArrayIndexOutOfBoundsException(index);

	 int j = elementCount - index - 1;
	 if (j > 0)
	  {
	    copy(elementData, index + 1, elementData, index, j);
	  }
	 elementCount--;
	 elementData[elementCount] = false; /* to let gc do its work */
    }

    /**
     * Inserts the specified object as a component in this VectorBoolean at the
     * specified <code>index</code>. Each component in this VectorBoolean with
     * an index greater or equal to the specified <code>index</code> is
     * shifted upward to have an index one greater than the value it had
     * previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than or equal to the current size of the VectorBoolean.
     */

    public final synchronized void insertElementAt(boolean obj, int index) {
	if (index >= elementCount + 1)
     throw new ArrayIndexOutOfBoundsException(index + " > " + elementCount);
     
	ensureCapacity(elementCount + 1);
	copy(elementData, index, elementData, index + 1, elementCount - index);
	elementData[index] = obj;
	elementCount++;
    }

    /**
     * Adds the specified component to the end of this VectorBoolean,
     * increasing its size by one. The capacity of this VectorBoolean is
     * increased if its size becomes greater than its capacity.
     */

     public final synchronized void addElement(boolean b)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = b;
     }


   /** Removes all components from this VectorBoolean  */

   public final synchronized void removeAllElements()
    {
	 for (int i = 0; i < elementCount; i++)
	  elementData[i] = false;

	 elementCount = 0;
    }
    

    
    static public void copy(boolean [] src, boolean [] dst, int len)
    {
     for (int i = 0; i < len; i++)  dst[i] = src[i];
     //System.arraycopy(src, 0, dst, 0, len);     
    }

    static public void copy(boolean [] src, int sdeb, boolean [] dst, int ddeb, int len)
    {
     for (int i = 0; i < len; i++)  dst[i + ddeb] = src[i + sdeb];
     //System.arraycopy(src, sdeb, dst, ddeb, len);          
    }
    
    
    // Enumerateur

    public int index = 0;

    public void restart() { index = 0; }
    
    public boolean hasMoreElements() { return index < elementCount; }    

    public boolean nextElement()
    {
     if (index < elementCount) return elementData[index++];
     else throw new NoSuchElementException("BooleanEnumerator");
    }
    
    

	//{{DECLARE_CONTROLS
	//}}
}
