/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import common.*;

public class VectorLong
{

    public long elementData[];
    public int elementCount;
    public int capacityIncrement;

    // construteurs
    public VectorLong(int initialCapacity, int capacityIncrement)
    {
	 super();
	 this.elementData = new long[initialCapacity];
	 this.capacityIncrement = capacityIncrement;
    		//{{INIT_CONTROLS
		//}}
}

    public VectorLong(int initialCapacity) { this(initialCapacity, 1); }

    public VectorLong() { this(16); }

    /**
     * Copies the components of this VectorLong into the specified array.
     * The array must be big enough to hold all the objects in this  VectorLong.
     */

    public final synchronized void copyInto(long anArray[])
    {
	 int i = elementCount;
	 
	 while (i-- > 0) {  anArray[i] = elementData[i]; }
    }

    /**
     * Trims the capacity of this VectorLong to be the VectorLong's current
     * size. An application can use this operation to minimize the
     * storage of a VectorLong.
     */

    public final synchronized void trimToSize()
    {
	 long oldCapacity = elementData.length;
	 if (elementCount < oldCapacity)
	  {
	    long oldData[] = elementData;
	    elementData = new long[elementCount];
	    copy(oldData, elementData, elementCount);
	  }
    }

    /**
     * Increases the capacity of this VectorLong, if necessary, to ensure
     * that it can hold at least the number of components specified by
     * the minimum capacity argument.
     */

    public final synchronized void ensureCapacity(int minCapacity)
    {
	 int oldCapacity = elementData.length;
	 if (minCapacity > oldCapacity)
	  {
	    long oldData[] = elementData;
	    int newCapacity = (capacityIncrement > 0) ?
		(oldCapacity + capacityIncrement) : (oldCapacity * 2);

        if (newCapacity < minCapacity) { newCapacity = minCapacity; }

	    elementData = new long[newCapacity];
        copy(oldData, elementData, elementCount);	    
	 }
    }

    /**
     * Sets the size of this VectorLong. If the new size is greater than the
     * current size, new <code>null</code> items are added to the end of
     * the VectorLong. If the new size is less than the current size, all
     * components at index <code>newSize</code> and greater are discarded.
     */

    public final synchronized void setSize(int newSize)
    {
	 if (newSize > elementCount) { ensureCapacity(newSize); }
	 else
	  {
	    for (int i = newSize ; i < elementCount ; i++)
		  elementData[i] = 0;
	  }
	 elementCount = newSize;
    }

    /** Returns the current capacity of this VectorLong */

    public final int capacity() { return elementData.length; }

    /** Returns the number of components in this VectorLong */

    public final int size() { return elementCount; }

    /** Tests if this VectorLong has no components  */

    public final boolean isEmpty() { return elementCount == 0; }

   /** Tests if the specified object is a component in this VectorLong  */

    public final boolean contains(long elem) { return indexOf(elem, 0) >= 0; }

    /** Searches for the first occurence of the given argument */

    public final int indexOf(long elem) { return indexOf(elem, 0); }

    public final synchronized int indexOf(long elem, int index)
    {

	 for (int i = index ; i < elementCount ; i++)
	   if (elem == elementData[i]) return i;

     return -1;
    }

    /** Returns the index of the last occurrence of the specified object */

    public final int lastIndexOf(long elem)
    {
	   return lastIndexOf(elem, elementCount-1);
    }

    /**
     * Searches backwards for the specified object, starting from the
     * specified index, and returns an index to it.
     */

    public final synchronized int lastIndexOf(long elem, int index)
    {
      for (int i = index ; i >= 0 ; i--)
      {
	    if (elem == elementData[i]) return i;
      }
	  return -1;
    }

    /** Returns the component at the specified index  */

    public final synchronized long elementAt(int index)
    {
	  if (index >= elementCount)
	    throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

      return elementData[index];
    }

    /** Returns the first component of this VectorLong  */

    public final synchronized long firstElement()
    {
	 if (elementCount == 0)  throw new NoSuchElementException();

	 return elementData[0];
    }

    /** Returns the last component of the VectorLong  */

    public final synchronized long lastElement()
    {
	 if (elementCount == 0) throw new NoSuchElementException();
	 return elementData[elementCount - 1];
    }

    /**
     * Sets the component at the specified <code>index</code> of this
     * VectorLong to be the specified object. The previous component at that
     * position is discarded.
     */
    public final synchronized void setElementAt(long obj, int index)
    {
 	 if(index >= elementCount)
	   throw new ArrayIndexOutOfBoundsException(index + " >= " + elementCount);

	 elementData[index] = obj;
    }

    /**
     * Deletes the component at the specified index. Each component in
     * this VectorLong with an index greater or equal to the specified
     * <code>index</code> is shifted downward to have an index one
     * smaller than the value it had previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than the current size of the VectorLong.
     */

    public final synchronized void removeElementAt(int index)
    {
	 if (index >= elementCount)
	 {
      throw new ArrayIndexOutOfBoundsException(index + " >= " +  elementCount);
	 }
	 else
	 if (index < 0)
	 {
	  throw new ArrayIndexOutOfBoundsException(index);
	 }

	 int j = elementCount - index - 1;
	 if (j > 0) copy(elementData, index + 1, elementData, index, j);
	 elementCount--;
	 elementData[elementCount] = 0; /* to let gc do its work */
    }

    /**
     * Inserts the specified object as a component in this VectorLong at the
     * specified <code>index</code>. Each component in this VectorLong with
     * an index greater or equal to the specified <code>index</code> is
     * shifted upward to have an index one greater than the value it had
     * previously.
     * <p>
     * The index must be a value greater than or equal to <code>0</code>
     * and less than or equal to the current size of the VectorLong.
     */

    public final synchronized void insertElementAt(long obj, int index)
    {
	 if(index >= elementCount + 1) 
	  throw new ArrayIndexOutOfBoundsException(index + " > " + elementCount);

	 ensureCapacity(elementCount + 1);
	 copy(elementData, index, elementData, index + 1, elementCount - index);
	 elementData[index] = obj;
	 elementCount++;
    }

    /**
     * Adds the specified component to the end of this VectorLong,
     * increasing its size by one. The capacity of this VectorLong is
     * increased if its size becomes greater than its capacity.
     */

     public final synchronized void addElement(long obj)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = obj;
     }

    /**
     * Removes the first occurrence of the argument from this VectorLong. If
     * the object is found in this VectorLong, each component in the VectorLong
     * with an index greater or equal to the object's index is shifted
     * downward to have an index one smaller than the value it had previously.
     */

    public final synchronized boolean removeElement(long obj)
    {
 	 int i = indexOf(obj);
	 if (i >= 0) { removeElementAt(i); return true; }
     return false;
    }

    /**
     * Removes all components from this VectorLong and sets its size to zero.
     */

    public final synchronized void removeAllElements()
    {
	 for (int i = 0; i < elementCount; i++)  elementData[i] = 0;
	 elementCount = 0;
    }




    /**
     * Returns a string representation of this VectorLong.
     */

    public final synchronized String toString()
    {
	 int max = size() - 1;
	 StringBuffer buf = new StringBuffer();

	 buf.append("[");

	 for (int i = 0 ; i <= max ; i++)
	 {
	   buf.append( (new Long(elementData[i])).toString());
	   if (i < max) { buf.append(", ");	 }
 	 }
	 
	 buf.append("]");
	 
	 return buf.toString();
    }
    
    
    public void copyFrom(long[] src)
    {
      int l = src.length;
      for(int i = 0; i < l; i++) elementData[i] = src[i];
    }    

        
    public void copyFrom(VectorLong src)
    {
      int l = src.size();
      for(int i = 0; i < l; i++) elementData[i] = src.elementAt(i);
    }    
        
        
        
    static public void copy(long [] src, long [] dst, int len)
    {
     for (int i = 0; i < len; i++)  dst[i] = src[i];
     //System.arraycopy(src, 0, dst, 0, len);     
    }

    static public void copy(long [] src, int sdeb, long [] dst, int ddeb, int len)
    {
     for (int i = 0; i < len; i++)  dst[i + ddeb] = src[i + sdeb];
     //System.arraycopy(src, sdeb, dst, ddeb, len);          
    }

    
        // Enumerateur

    public int index = 0;

    public void restart() { index = 0; }
    
    public boolean hasMoreElements() { return index < elementCount; }    

    public long nextElement()
    {
     if (index < elementCount) return elementData[index++];
     else throw new NoSuchElementException("LongEnumerator");
    }

	//{{DECLARE_CONTROLS
	//}}
}
